/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * Purpose: Extract the color index data from the bitmap file "DigitalCamouflage.png" and save it to the file      * */
/* * "DigitalCamouflage.inc"                                                                                         * */
/* *                                                                                                                 * */
/* * Usage: Create a three-color digital camouflage bitmap image in your desired dimensions. Determine the rgb       * */
/* * values of the three camouflage colors and enter them in the "PixelColor" array below. Run this file. A new file * */
/* * called "DigitalCamouflage.inc" will be created. This file will contain a two-dimensional array of pixel color   * */
/* * indexes, which you can then use in whatever manner you wish.                                                    * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

//-f

#version 3.7;

global_settings {assumed_gamma 2.2}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * SET THESE VARIABLES                                                                                             * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#local SourceBitmapFileName = "DigitalCamouflage.png"; //Bitmap image to READ

#local DataFileName = "DigitalCamouflage.inc"; //Data File to WRITE

#local NPixelColors = 3; //Number of colors used in your bitmap image

#local PixelColor = array [NPixelColors] { //These rgb values need to match the colors used in the bitmap image
   <206, 193, 177> / 255,
   <153, 146, 128> / 255,
   <100, 103, 94> / 255
} //array

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * CAMERA                                                                                                          * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

camera {
   orthographic
   location <0, 0, -.75>
   direction <0, 0, 1>
   look_at <0, 0, 0>
} //camera

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * EVAL_PIGMENT () MACRO                                                                                           * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#macro eval_pigment (PigmentInQuestion, Location) //From functions.inc

   #local fn = function {pigment {PigmentInQuestion}}

   #local Result = (fn (Location.x, Location.y, Location.z));

   Result

#end //#macro eval_pigment

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * DISPLAY THE SOURCE IMAGE                                                                                        * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#local SourceBitmapImagePigment = pigment {image_map {png SourceBitmapFileName gamma 2.2 map_type 0 once}}

#local SourceBitmapImageResolution = max_extent (SourceBitmapImagePigment);

#debug concat ("\nSource resolution: ", str (SourceBitmapImageResolution.x, 0, 0), " x ", str (SourceBitmapImageResolution.y, 0, 0), "\n")

#local SourceBitmapImagePigment = pigment {SourceBitmapImagePigment scale <SourceBitmapImageResolution.x, SourceBitmapImageResolution.y, 1>}

#local SourceImage = object {
   box {<0, 0, -.1> <SourceBitmapImageResolution.x, SourceBitmapImageResolution.y, 0> texture {pigment {SourceBitmapImagePigment} finish {emission 1}}}
} //object

#local SourceImage = object {
   #if (SourceBitmapImageResolution.x > SourceBitmapImageResolution.y)
      #local Denominator = SourceBitmapImageResolution.x;
   #else
      #local Denominator = SourceBitmapImageResolution.y;
   #end //#if
   SourceImage
   scale <1/Denominator, 1/Denominator, 1>
} //object

object {SourceImage translate <-SourceBitmapImageResolution.x / Denominator / 2, -SourceBitmapImageResolution.y / Denominator / 2, 0>}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * CREATE A 2-D ARRAY OF POINTS                                                                                    * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#for (I, 0, NPixelColors - 1)
   #local PixelColor [I] = <(int (PixelColor [I].red * 1000 + .5)) / 1000, (int (PixelColor [I].green * 1000 + .5)) / 1000, (int (PixelColor [I].blue * 1000 + .5)) / 1000>;
#end //#for

#local PixelColorIndex = array [SourceBitmapImageResolution.x] [SourceBitmapImageResolution.y]

#for (XIndex, 0, SourceBitmapImageResolution.x - 1)
   #for (YIndex, 0, SourceBitmapImageResolution.y - 1)
      //Adjust sampling point so that integer <x, y> coordinates indicate the center of a pixel.
      #local CurX = XIndex + .5;
      #local CurY = YIndex + .5;
      #local CurPixelPigment = eval_pigment (SourceBitmapImagePigment, <CurX, CurY, 0>);
      #local CurPixelPigment = <(int (CurPixelPigment.red * 1000 + .5)) / 1000, (int (CurPixelPigment.green * 1000 + .5)) / 1000, (int (CurPixelPigment.blue * 1000 + .5)) / 1000>;
      #local ColorIndexFound = false;
      #for (I, 0, NPixelColors - 1)
         #if (CurPixelPigment.red = PixelColor [I].red & CurPixelPigment.green = PixelColor [I].green & CurPixelPigment.blue = PixelColor [I].blue)
            #local PixelColorIndex [XIndex] [YIndex] = I;
            #local ColorIndexFound = true;
            #break
         #end //#if
      #end //#for
      #if (!ColorIndexFound)
         #debug concat ("\nThe pixel at (", str (XIndex, 0, 0), ", ", str (YIndex, 0, 0), ") did not match any defined colors.\n")
         #debug concat ("The rgb values of the pixel are <", str (CurPixelPigment.red, 0, 3), ", ", str (CurPixelPigment.green, 0, 3), ", ", str (CurPixelPigment.blue, 0, 3), ">.\n")
         #error concat ("\nProcessing halted.\n")
      #end //#if
   #end //#for
#end //#for

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * WRITE THE COLOR INDEX DATA TO A FILE                                                                            * */
/* *                                                                                                                 * */
/* *                                                                                                                 * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#debug concat ("\nWriting Data File:  \"", DataFileName, "\"\n\n")

#fopen DataFile DataFileName write

#write (DataFile, "#local NPixels_X = ", str (SourceBitmapImageResolution.x, 0, 0), ";\n")
#write (DataFile, "#local NPixels_Y = ", str (SourceBitmapImageResolution.y, 0, 0), ";\n\n")

#write (DataFile, "#local PixelColorIndex = array [NPixels_X] [NPixels_Y] {\n")

#for (I, 0, SourceBitmapImageResolution.x - 1)
   #write (DataFile, "   {")
   #for (J, 0, SourceBitmapImageResolution.y - 1)
      #write (DataFile, str (PixelColorIndex [I] [J], 0, 0))
      #if (J < SourceBitmapImageResolution.y - 1)
         #write (DataFile, ", ")
      #end //#if
   #end //#for
   #if (I < SourceBitmapImageResolution.x - 1)
      #write (DataFile, "},\n")
   #else
      #write (DataFile, "}\n")
   #end //#if
#end //#for

#write (DataFile, "} //array\n")

#fclose DataFile
