/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                                                                                 * */
/* * This file will create a three-color digital camouflage pattern. The variable "ScaleFactor" is related to the    * */
/* * final output resolution, which should be 800 x 800. Changing one will require a proportional change to the      * */
/* * other.                                                                                                          * */
/* *                                                                                                                 * */
/* * Anti-aliasing must be turned off ("-a") to achieve correct results.                                             * */
/* *                                                                                                                 * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

//+w800 +h800 -a

#version 3.7;

global_settings {assumed_gamma 2.2}

camera {
   orthographic
   location <0, 0, -1>
   right 1 * x
   direction <0, 0, 1>
   look_at <0, 0, 0>
} //camera

#local ScaleFactor = .03;

#local POVCamoPigment0 = color <206, 193, 177> / 255 * .902; //Background color
#local POVCamoPigment1 = color <153, 146, 128> / 255 * .902; //I don't know why the colors have to be dimmed. I'm
#local POVCamoPigment2 = color <100, 103, 94> / 255 * .907;  //guessing it's probably a gamma issue.

#local CrossoverPoint0 = .45;
#local CrossoverPoint1 = .50;

#local POVCamoLayer0 = texture {
   pigment {
      granite
      warp {turbulence .1 lambda 5 omega .75}
      color_map {
         [0 color POVCamoPigment0]
         [CrossoverPoint0 color POVCamoPigment0]
         [CrossoverPoint0 color POVCamoPigment1]
         [1 color POVCamoPigment1]
      } //color_map
   } //pigment
   finish {emission 1}
} //texture

#local POVCamoLayer1 = texture {
   pigment {
      granite
      warp {turbulence .1 lambda 5 omega .75}
      color_map {
         [0 color rgbt <1, 1, 1, 1>]
         [CrossoverPoint1 color rgbt <1, 1, 1, 1>]
         [CrossoverPoint1 color POVCamoPigment2]
         [1 color POVCamoPigment2]
      } //color_map
      translate 10 * z
   } //pigment
   finish {emission 1}
} //texture

#local POVCamoColor = texture {POVCamoLayer0} texture {POVCamoLayer1}

#local POVCamo = object {
   plane {
      z, 0 inverse
      texture {
         POVCamoColor
         scale <ScaleFactor, ScaleFactor, 1>
      } //texture
   } //plane
} //object

object {POVCamo}
